#ifndef __MULTI_STAGE_H
#define __MULTI_STAGE_H

#include "orbitersdk.h"
#include <math.h>
#include <stdio.h>
#include <fstream.h>

const double G   = 9.80665;
const double N   = 1.0;
const double kN  = 1000.0;
const double KGF = N*G;
const double SEC = 1.0*G;
const double KG  = 1.0;
const double PI  = 3.14159265359;
const double RAD = PI/180.0;

double GetCurrentValue(VESSEL *vessel, double sl, double vac)
{
	double c_atm=0;
	double c_value=0;
	c_atm = vessel->GetAtmPressure() / 101.4e3; 
	c_value = c_atm * (sl-vac);				
	c_value += vac;								
	return c_value;
}

void DoThrustAdjust(VESSEL *vessel, double sealevel, double vacuum)
{
	double *booster_thrust;
	booster_thrust = vessel->GetMainThrustModPtr();
	*booster_thrust = 0;
	*booster_thrust = (GetCurrentValue(vessel,sealevel,vacuum)-GetCurrentValue(vessel,vacuum,vacuum));
}


double CalcSeaLevelThrust(double vacThrust, double vacISP, double slISP, double FuelMass)
{
	double vacBurnTime = 0.0;
	vacBurnTime = (FuelMass / vacThrust) * vacISP * G;
	return (FuelMass / vacBurnTime) * slISP *G;
}


void DoReEntryFlames(VESSEL *vessel, int ReEntryID, double width)
{
	double airspeed = 0.0;
	double factor = 0.0;
	double airdensity = 0.0;
	double widthfactor = 0.0;
	oapiGetFocusAirspeed(&airspeed);
	airdensity = vessel->GetAtmDensity();
	factor = .5 * airdensity * (airspeed * airspeed);
	factor = factor - 75000;
	if (factor < 0)
		factor = 0;
	factor = factor / 25000;
	if (factor)
		vessel->SetExhaustScales (EXHAUST_CUSTOM, ReEntryID, factor, width);
	else
		vessel->SetExhaustScales (EXHAUST_CUSTOM, ReEntryID, 0, 0);

}

double GetFuelUsage(double simt, double thrust, double ISP)
{
	return (simt * thrust) / ISP;
}

int Booster_nt = 6;
double Booster_Seq[6]    = {0,0,0,0,0,0};
double Booster_Thrust[6] = {0,                     1,     1,   0.8,               0.15,  0};
double Booster_Prop[6]   = {1,               0.99583,0.1618,0.0492,            0.00563,  0};
double Booster_ThrSCL[5] = {0,0,0,0,0};
double Booster_PrpSCL[5] = {0,0,0,0,0};

void Booster_Setup(double booster_sep_time, double booster_fuel_mass)
{
	Booster_Seq[0]= 0;
	Booster_Seq[1]= 1;
	Booster_Seq[2]= booster_sep_time-24;
	Booster_Seq[3]=	booster_sep_time-9;
	Booster_Seq[4]= booster_sep_time;
	Booster_Seq[5]= booster_sep_time+9;
	for (int i = 0; i < 5; i++)
	{
		Booster_ThrSCL[i]=(Booster_Thrust[i+1]-Booster_Thrust[i])/(Booster_Seq[i+1]-Booster_Seq[i]);
		Booster_PrpSCL[i]=(Booster_Prop[i+1]-Booster_Prop[i])/(Booster_Seq[i+1]-Booster_Seq[i]);
	}
}

void GetBooster_State (double srb_time, double &thrust_level, double &prop_level)
{
	for (int i = Booster_nt-2; i >= 0; i--)
		if (srb_time > Booster_Seq[i]) break;
	thrust_level = Booster_ThrSCL[i] * (srb_time-Booster_Seq[i]) + Booster_Thrust[i];
	prop_level   = Booster_PrpSCL[i] * (srb_time-Booster_Seq[i]) + Booster_Prop[i];
}


void StageTransform(VESSEL *vessel, VESSELSTATUS *vs, VECTOR3 ofs, VECTOR3 vel)
{
	VECTOR3 rofs, rvel = {vs->rvel.x, vs->rvel.y, vs->rvel.z};
// Staging Velocity represents 
// Need to do some transforms to get the correct vector to eject the stage
	vessel->Local2Rel (ofs, vs->rpos);
	vessel->GlobalRot (vel, rofs);
	vs->rvel.x = rvel.x+rofs.x;
	vs->rvel.y = rvel.y+rofs.y;
	vs->rvel.z = rvel.z+rofs.z;
}

// ==========================================================
// Interface for derived vessel class: Atlantis
// ==========================================================

class IStage
{
public:
	virtual void Init ();
	virtual void Exit (VESSEL *vessel);
	virtual void Setup (VESSEL *vessel, double stage_sep_time);
	virtual void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	virtual void SaveState (VESSEL *vessel, FILEHANDLE scn);
	virtual void Timestep (VESSEL *vessel, double simt);
	virtual void Animate (VESSEL *vessel, double simt);
	virtual void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	virtual void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	virtual int ConsumeKey (VESSEL *vessel, const char *keystate);
};

void IStage::Init ()
{
};
void IStage::Exit (VESSEL *vessel)
{
};
void IStage::Setup (VESSEL *vessel, double stage_sep_time)
{
};
void IStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
};
void IStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
};
void IStage::Timestep (VESSEL *vessel, double simt)
{
};
void IStage::Animate (VESSEL *vessel, double simt)
{
};
void IStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void IStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int IStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	return 0;
};

class Multi_Stage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void SetClassCaps (VESSEL *vessel, FILEHANDLE cfg);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
	void SetStage (IStage *Stage, VESSEL *vessel, double simt);
	void SetCurrentStage(IStage *Stage);
	VISHANDLE hVis;
private:
	IStage *cStage;
};


void Multi_Stage::Init ()
{
	if (cStage)
		cStage->Init();
};
void Multi_Stage::Exit (VESSEL *vessel)
{
	if (cStage)
		cStage->Exit(vessel);
};
void Multi_Stage::SetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	if (cStage)
		cStage->Init();
};
void Multi_Stage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	if (cStage)
		cStage->ParseLine(vessel,line,def_vs);
};
void Multi_Stage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	if (cStage)
		cStage->SaveState(vessel,scn);
};
void Multi_Stage::Timestep (VESSEL *vessel, double simt)
{
	if (cStage)
		cStage->Timestep(vessel,simt);
};
void Multi_Stage::Animate (VESSEL *vessel, double simt)
{
	if (cStage)
		cStage->Animate(vessel,simt);
};
void Multi_Stage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
	if (refcount > 1) return; 
	hVis=vis;
	if (cStage)
		cStage->VisualCreated(vessel,vis,refcount);
	Animate(vessel,0);
};
void Multi_Stage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
	hVis=NULL;
	if (cStage)
		cStage->VisualDestroyed(vessel,vis,refcount);
};
int Multi_Stage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (cStage)
		return cStage->ConsumeKey(vessel,keystate);
	return 0;
};
void Multi_Stage::SetStage (IStage *Stage, VESSEL *vessel, double simt)
{
	double *booster;
	cStage=Stage;
	if (cStage)
	{
		cStage->Init();
		cStage->Setup(vessel,simt);
		booster = vessel->GetMainThrustModPtr();
		*booster = 0;

	};
};
void Multi_Stage::SetCurrentStage (IStage *Stage)
{
	cStage=Stage;
};


Multi_Stage *Stack;
#endif // !__MULTI_STAGE_H

